/*
 *  neercs        console-based window manager
 *  Copyright (c) 2006-2010 Sam Hocevar <sam@hocevar.net>
 *                2008-2010 Jean-Yves Lamoureux <jylam@lnxscene.org>
 *                2008-2010 Pascal Terjan <pterjan@linuxfr.org>
 *                All Rights Reserved
 *
 *  This program is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#if defined HAVE_CONFIG_H
#   include "config.h"
#endif

#include <stdio.h> /* perror() */
#include <stdlib.h> /* malloc(), free() */
#include <unistd.h> /* unlink() */
#include <fcntl.h> /* fcntl() */
#include <string.h> /* memcpy() */
#include <sys/select.h> /* select() */
#include <sys/types.h> /* bind(), connect() */
#include <sys/socket.h> /* bind(), connect() */
#include <sys/un.h> /* AF_UNIX */

#include "mini-neercs.h"
#include "mini-socket.h"

#define SIZEOF_SUN_PATH (sizeof(((struct sockaddr_un *)NULL)->sun_path))

struct nrx_socket
{
#if 1
    /* Linux sockets */
    int fd;
    int server;
    char path[SIZEOF_SUN_PATH];
#else
#   error No socket implementation
#endif
};

nrx_socket_t * socket_open(char const *path, int server)
{
    nrx_socket_t * sock;
    struct sockaddr_un addr;
    int ret, fd;

    fd = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        perror("socket creation");
        return NULL;
    }

    memset(&addr, 0, sizeof(struct sockaddr_un));
    addr.sun_family = AF_UNIX;
    strncpy(addr.sun_path, path, SIZEOF_SUN_PATH - 1);

    if (server)
    {
        unlink(path);
        ret = bind(fd, (struct sockaddr *)&addr, sizeof(addr));
    }
    else
    {
        ret = connect(fd, (struct sockaddr *)&addr, sizeof(addr));
    }

    if (ret < 0)
    {
        perror(server ? "socket binding" : "socket connection");
        close(fd);
        return NULL;
    }

    fcntl(fd, F_SETFL, O_NONBLOCK);

    sock = malloc(sizeof(*sock));
    sock->fd = fd;
    sock->server = server;
    strncpy(sock->path, path, SIZEOF_SUN_PATH - 1);

    return sock;
}

int socket_select(nrx_socket_t *sock, int usecs)
{
    fd_set rfds;
    struct timeval tv;
    int ret;

    FD_ZERO(&rfds);
    FD_SET(sock->fd, &rfds);

    tv.tv_sec = usecs / 1000000;
    tv.tv_usec = usecs % 1000000;

    ret = select(sock->fd + 1, &rfds, NULL, NULL, &tv);
    if (ret < 0)
        return -1;

    if (FD_ISSET(sock->fd, &rfds))
        return 1;

    return 0;
}

int socket_puts(nrx_socket_t *sock, char const *str)
{
    int ret;
    ret = write(sock->fd, str, strlen(str));
    return ret;
}

ssize_t socket_read(nrx_socket_t *sock, void *buf, size_t count)
{
    int ret;
    ret = read(sock->fd, buf, count);
    return ret;
}

void socket_close(nrx_socket_t *sock)
{
    close(sock->fd);
    if (sock->server)
        unlink(sock->path);
    free(sock);
}

